Package: HiCDOC
Title: A/B compartment detection and differential analysis
Version: 1.15.2
Date: 2025-07-22
Authors@R: c(
    person("Kurylo", "Cyril", email = "cyril.kurylo@inrae.fr", role = c("aut")),
    person("Zytnicki", "Matthias", email = "matthias.zytnicki@inrae.fr", role = c("aut")),
    person("Foissac", "Sylvain", email = "sylvain.foissac@inrae.fr", role = c("aut")),
    person("Maigné", "Élise", email = "elise.maigne@inrae.fr", role = c("aut", "cre")))
Description: HiCDOC normalizes intrachromosomal Hi-C matrices, uses
        unsupervised learning to predict A/B compartments from multiple
        replicates, and detects significant compartment changes between
        experiment conditions. It provides a collection of functions
        assembled into a pipeline to filter and normalize the data,
        predict the compartments and visualize the results. It accepts
        several type of data: tabular `.tsv` files, Cooler `.cool` or
        `.mcool` files, Juicer `.hic` files or HiC-Pro `.matrix` and
        `.bed` files.
License: LGPL-3
Encoding: UTF-8
Depends: InteractionSet, GenomicRanges, SummarizedExperiment, R (>=
        4.1.0)
Imports: methods, ggplot2, Rcpp (>= 0.12.8), stats, S4Vectors, gtools,
        pbapply, BiocParallel, BiocGenerics, grid, cowplot, gridExtra,
        data.table, multiHiCcompare, Seqinfo, HiCParser
LinkingTo: Rcpp
Suggests: knitr, rmarkdown, testthat, BiocStyle, BiocManager, rhdf5
biocViews: HiC, DNA3DStructure, Normalization, Sequencing, Software,
        Clustering
VignetteBuilder: knitr
URL: https://github.com/mzytnicki/HiCDOC
BugReports: https://github.com/mzytnicki/HiCDOC/issues
Config/roxygen2/version: 8.0.0
Config/pak/sysreqs: libssl-dev zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-06-08 14:08:02 UTC
RemoteUrl: https://github.com/bioc/HiCDOC
RemoteRef: HEAD
RemoteSha: 2c0c315182e4170e5604b42608222fb89a7b6a9a
NeedsCompilation: yes
Packaged: 2026-06-08 22:32:04 UTC; root
Author: Kurylo Cyril [aut],
  Zytnicki Matthias [aut],
  Foissac Sylvain [aut],
  Maigné Élise [aut, cre]
Maintainer: Maigné Élise <elise.maigne@inrae.fr>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-06-08 22:35:08 UTC; unix
