# Generated by roxygen2: do not edit by hand

export("parameters<-")
export(HiCDOC)
export(HiCDOCDataSetFromCool)
export(HiCDOCDataSetFromHiC)
export(HiCDOCDataSetFromHiCPro)
export(HiCDOCDataSetFromTabular)
export(chromosomes)
export(compartments)
export(concordances)
export(defaultHiCDOCParameters)
export(detectCompartments)
export(differences)
export(filterSmallChromosomes)
export(filterSparseReplicates)
export(filterWeakPositions)
export(normalizeBiologicalBiases)
export(normalizeDistanceEffect)
export(normalizeTechnicalBiases)
export(parameters)
export(plotCentroids)
export(plotCompartmentChanges)
export(plotCompartments)
export(plotConcordanceDifferences)
export(plotConcordances)
export(plotDistanceEffect)
export(plotInteractions)
export(plotSelfInteractionRatios)
export(reduceHiCDOCDataSet)
export(sampleConditions)
export(sampleReplicates)
exportClasses(HiCDOCDataSet)
exportMethods("parameters<-")
exportMethods(chromosomes)
exportMethods(compartments)
exportMethods(concordances)
exportMethods(differences)
exportMethods(parameters)
exportMethods(sampleConditions)
exportMethods(sampleReplicates)
exportMethods(show)
import(HiCParser)
import(InteractionSet)
import(data.table)
import(ggplot2)
import(methods)
importFrom(BiocGenerics,cbind)
importFrom(BiocGenerics,width)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpmapply)
importFrom(BiocParallel,bpparam)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,distance)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,match)
importFrom(GenomicRanges,start)
importFrom(GenomicRanges,union)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,"%in%")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,runLength)
importFrom(S4Vectors,split)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqnames)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(cowplot,get_plot_component)
importFrom(cowplot,ggdraw)
importFrom(grid,gpar)
importFrom(grid,textGrob)
importFrom(gridExtra,arrangeGrob)
importFrom(gtools,mixedsort)
importFrom(multiHiCcompare,cyclic_loess)
importFrom(multiHiCcompare,hic_table)
importFrom(multiHiCcompare,make_hicexp)
importFrom(pbapply,pbmapply)
importFrom(stats,ecdf)
importFrom(stats,loess)
importFrom(stats,loess.control)
importFrom(stats,median)
importFrom(stats,optimize)
importFrom(stats,p.adjust)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,update)
importFrom(stats,wilcox.test)
useDynLib(HiCDOC)
