# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

constrainedClustering <- function(rMatrix, rLinks, maxDelta = 0.0001, maxIterations = 50L, totalRestarts = 20L, totalClusters = 2L) {
    .Call('_HiCDOC_constrainedClustering', PACKAGE = 'HiCDOC', rMatrix, rLinks, maxDelta, maxIterations, totalRestarts, totalClusters)
}

